import {
  expose,
  Icon,
  IconEnum,
  List,
  Markdown,
  TemplateUiCommand,
  ui,
} from "@kksh/api/ui/template";
import type {
  ApiResponse,
  OnThisDayJson,
  WikipediaOnThisDay,
  WikipediaPageJson,
} from "./types";
import type { WikipediaPage } from "./types";

class OnThisDay extends TemplateUiCommand {
  private wikipediaPages: WikipediaOnThisDay[] = [];

  detailViewOfOnThisDay(onThisDay: WikipediaOnThisDay, date: Date) {
    const dateMarkdown =
      "## " + this.dateGetMonthAndDay(date) + ", " + onThisDay.year.toString();
    const eventDetails = "### " + onThisDay.eventDescription;
    const relatedArticlesMarkdown =
      "### " + `Related Articles (${onThisDay.pages.length.toString()})`;
    const pageDataMarkdown = onThisDay.pages
      .map(
        (page: WikipediaPage) =>
          `### [${page.title}](${page.pageUrl})
          \n\n${page.extract}
          \n\n![Image](${page.imageUrl})\n`
      )
      .join("\n---\n");

    return ui.render(
      new Markdown(
        dateMarkdown +
          "\n" +
          eventDetails +
          "\n" +
          relatedArticlesMarkdown +
          "\n" +
          pageDataMarkdown
      )
    );
  }

  dateGetMonthAndDay(date: Date) {
    const monthString = date.toLocaleString("default", { month: "long" });
    const day = date.getDate();
    return `${monthString} ${day}`;
  }

  async getWikipediaOnThisDay(date: Date): Promise<WikipediaOnThisDay[]> {
    try {
      const year = date.getFullYear();
      let month: string | number = date.getMonth() + 1;
      let day: string | number = date.getDate();
      month = month < 10 ? "0" + month : month.toString();
      day = day < 10 ? "0" + day : day.toString();
      const response = await fetch(
        `https://api.wikimedia.org/feed/v1/wikipedia/en/featured/${year}/${month}/${day}`
      );
      const data = (await response.json()) as ApiResponse;
      const onThisDay = data.onthisday;
      const wikipediaPages: WikipediaOnThisDay[] = onThisDay.map(
        (item: OnThisDayJson) => {
          const subpages: WikipediaPage[] = item.pages.map(
            (subItem: WikipediaPageJson) => ({
              title: subItem.normalizedtitle,
              extract: subItem.extract,
              pageUrl: subItem.content_urls.desktop.page,
              imageUrl: subItem.thumbnail?.source || "",
            })
          );
          // setIsLoading(false);
          return {
            year: item.year,
            eventDescription: item.text,
            pages: subpages,
          };
        }
      );

      return wikipediaPages;
    } catch (error) {
      console.error("Error loading Wikipedia stories", error);
      return [];
    }
  }

  async load() {
    const date = new Date();
    this.wikipediaPages = await this.getWikipediaOnThisDay(date);
    console.log(this.wikipediaPages);

    if (!this.wikipediaPages) {
      return ui.render(new Markdown("No data found"));
    }

    return ui
      .setSearchBarPlaceholder("Enter a search term, and press enter to search")
      .then(() => {
        return ui.render(
          new List.List({
            sections: [
              new List.Section({
                title: "On This Day in History",
                items: this.wikipediaPages.map(
                  (item) =>
                    new List.Item({
                      title: item.eventDescription,
                      value: item.eventDescription,
                      defaultAction: "Show Details",
                      accessories: [
                        new List.ItemAccessory({
                          date: date,
                          text: item.year.toString(),
                          tooltip: "Year",
                        }),
                        // new List.ItemAccessory({
                        //   text: "Pages: " + item.pages.length.toString(),
                        //   tooltip: "Pages",
                        // }),
                      ],
                      icon: new Icon({
                        type: IconEnum.Iconify,
                        value: "mdi:wikipedia",
                      }),
                    })
                ),
              }),
            ],
          })
        );
      });
  }

  onListItemSelected(value: string): Promise<void> {
    const wikipediaPage = this.wikipediaPages.find(
      (item) => item.eventDescription === value
    );

    if (!wikipediaPage) {
      return ui.render(new Markdown("No data found"));
    }

    return this.detailViewOfOnThisDay(wikipediaPage, new Date());
  }
}

expose(new OnThisDay());
